@@ -138,12 +138,7 @@ def to_single_stage_lockfile(stage: "Stage") -> dict:
     params, deps = split_params_deps(stage)
     deps, outs = [
         [
-            OrderedDict(
-                [
-                    (PARAM_PATH, item.def_path),
-                    (item.checksum_type, item.checksum),
-                ]
-            )
+            OrderedDict([(PARAM_PATH, item.def_path), *item.info.items()])
             for item in sort_by_path(items)
         ]
         for items in [deps, stage.outs]
