@@ -35,7 +35,7 @@ class StateNoop:
     def save(self, path_info, checksum):
         pass
 
-    def get(self, path_info):
+    def get(self, path_info):  # pylint: disable=unused-argument
         return None
 
     def save_link(self, path_info):
@@ -89,10 +89,12 @@ class State:  # pylint: disable=too-many-instance-attributes
     MAX_UINT = 2 ** 64 - 2
 
     def __init__(self, local_cache):
+        from dvc.tree.local import LocalTree
+
         repo = local_cache.repo
         self.repo = repo
         self.root_dir = repo.root_dir
-        self.tree = local_cache.tree.work_tree
+        self.tree = LocalTree(None, {"url": self.root_dir})
 
         state_config = repo.config.get("state", {})
         self.row_limit = state_config.get("row_limit", self.STATE_ROW_LIMIT)
@@ -394,8 +396,8 @@ class State:  # pylint: disable=too-many-instance-attributes
         assert isinstance(path_info, str) or path_info.scheme == "local"
         path = os.fspath(path_info)
 
-        # NOTE: use os.path.exists instead of WorkingTree.exists
-        # WorkingTree.exists uses lexists() and will return True for broken
+        # NOTE: use os.path.exists instead of LocalTree.exists
+        # because it uses lexists() and will return True for broken
         # symlinks that we cannot stat() in get_mtime_and_size
         if not os.path.exists(path):
             return None
@@ -445,9 +447,9 @@ class State:  # pylint: disable=too-many-instance-attributes
 
         self._execute(f"SELECT * FROM {self.LINK_STATE_TABLE}")
         for row in self.cursor:
-            relpath, inode, mtime = row
+            relative_path, inode, mtime = row
             inode = self._from_sqlite(inode)
-            path = os.path.join(self.root_dir, relpath)
+            path = os.path.join(self.root_dir, relative_path)
 
             if path in used or not self.tree.exists(path):
                 continue
@@ -457,7 +459,7 @@ class State:  # pylint: disable=too-many-instance-attributes
 
             if (inode, mtime) == (actual_inode, actual_mtime):
                 logger.debug("Removing '%s' as unused link.", path)
-                unused.append(relpath)
+                unused.append(relative_path)
 
         return unused
 
